/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef _EXCEPTIONS_H_
#define _EXCEPTIONS_H_
#include <iostream>
#include <string>
#include <signal.h>

#include <src/global/global.h>

#include <src/global/sources/position.h>

namespace Investigation
{
	namespace Global
	{
		namespace Exception
		{
			std::ostream& details(std::ostream& os);

			class Generique
			{
				public:
					Generique() = default;
					void format(std::ostream& os) const;
					void change_position(Investigation::Global::Source::PositionSP position)
					{
						if(position)
							_position = position;
					};
				protected:
					Generique(Investigation::Global::Source::PositionSP position, std::string message)
						:_position(position),_message(message) {};
					Generique(std::string message)
						:_message(message) {};
					Investigation::Global::Source::PositionSP _position;
					std::string _message;
					std::string message() const
					{
						if(_position)
							return std::string(*_position)+_message;
						return _message; 
					};
			};


			class Compilation : public Generique
			{
				protected:
					Compilation(Investigation::Global::Source::PositionSP position, std::string message)
						:Generique(position,std::string("Compilation error: ")+message) {};
			};

			class Execution : public Generique
			{
				public:
					Execution(size_t interruption, std::string message)
						:Generique(std::string("Runtime error (")+Execution::nom_interruption(interruption)+("): ")+message),_interruption(interruption) {};
					operator size_t() const
					{
						return _interruption;
					}
					static std::string nom_interruption(const size_t& signal)
					{
						switch(signal)
						{
							case SIGHUP:
								return "HUP";
							case SIGINT:	
								return "KBINT";
							case SIGQUIT:
								return "QUIT";
							case SIGILL:
								return "ILL";
							case SIGABRT:
								return "ABRT";
							case SIGFPE:
								return "FPE";
							case SIGSEGV:
								return "SEGV";
							case SIGPIPE:
								return "PIPE";
							case SIGALRM:
								return "ALRM";
							case SIGTERM:
								return "TERM";
							case SIGUSR1:
								return "USR1";
							case SIGUSR2:
								return "USR2";
							case SIGCHLD:
								return "CHLD";
							case SIGCONT:
								return "CONT";
							case SIGTSTP:
								return "TSTP";
							case SIGTTIN:
								return "TTIN";
							case SIGTTOU:
								return "TTOU";
							case SIGBUS:
								return "BUS";
							case SIGPROF:
								return "PROF";
							case SIGURG:
								return "URG";
							case SIGIO:
								return "IO";
							default:
								return "";
								break;
						}
					}
				private:
					size_t _interruption;
			};

			class ErreurDeSyntaxe : public Compilation
			{
				public:
					ErreurDeSyntaxe(Investigation::Global::Source::PositionSP position, std::string message)
						:Compilation(position,std::string("Syntax error.")) {};
			};

			class PasDeFichierSource : public Compilation
			{
				public:
					PasDeFichierSource(std::string fichier_source)
						:Compilation(Investigation::Global::Source::PositionSP(),std::string("Source file ")+fichier_source+std::string(" is invalid.")) {};
			};

			class InstructionIllegale : public Investigation::Global::Exception::Execution
			{
				public:
					InstructionIllegale(const std::string& instruction)
					:Execution(SIGILL,std::string("Illegal instruction ")+instruction) { }
			};

			class Integration : public Investigation::Global::Exception::Generique
			{
				protected:
					Integration(std::string message)
						:Generique(Investigation::Global::Source::PositionSP(),std::string("Startup error: ")+message) {};
			};

			class Adapteur : public Investigation::Global::Exception::Generique
			{
				protected:
					Adapteur(std::string message)
						:Generique(Investigation::Global::Source::PositionSP(),std::string("Wrapper error: ")+message) {};
			};
		}
	}
}
#endif
